diff mbox

Fix PR 52203

Message ID 4F574775.4040500@ispras.ru
State New
Headers show

Commit Message

Andrey Belevantsev March 7, 2012, 11:33 a.m. UTC
Hello,

This PR is again about insns that are recog'ed as >=0 but do not change the 
processor state.  As explained in 
http://gcc.gnu.org/bugzilla/show_bug.cgi?id=52203#c8, I've tried 
experimenting with an attribute marking those "empty" insns in MD files and 
asserting that all other insns do have reservations.  As this doesn't seem 
to be interesting, I give up with the idea, and the below patch makes 
sel-sched do exactly what the Haifa scheduler does, i.e. do not count such 
insns against issue_rate when modelling clock cycles.

Tested on ia64 and x86-64, OK for trunk?  No testcase again because of the 
amount of flags needed.

Andrey

2012-03-07  Andrey Belevantsev  <abel@ispras.ru>

	PR rtl-optimization/52203
	* sel-sched.c (estimate_insn_cost): New parameter pempty.  Adjust
	all callers to pass NULL except ...
	(reset_sched_cycles_in_current_ebb): ... here, save the value
	in new variable 'empty'.  Increase issue_rate only for
	non-empty insns.

    int cost;
@@ -4277,6 +4278,8 @@ estimate_insn_cost (rtx insn, state_t state)

    memcpy (temp, state, dfa_state_size);
    cost = state_transition (temp, insn);
+  if (pempty)
+    *pempty = (memcmp (temp, state, dfa_state_size) == 0);

    if (cost < 0)
      return 0;
@@ -4307,7 +4310,7 @@ get_expr_cost (expr_t expr, fence_t fence)
  	return 0;
      }
    else
-    return estimate_insn_cost (insn, FENCE_STATE (fence));
+    return estimate_insn_cost (insn, FENCE_STATE (fence), NULL);
  }

  /* Find the best insn for scheduling, either via max_issue or just take
@@ -7020,7 +7023,7 @@ reset_sched_cycles_in_current_ebb (void)
      {
        int cost, haifa_cost;
        int sort_p;
-      bool asm_p, real_insn, after_stall, all_issued;
+      bool asm_p, real_insn, after_stall, all_issued, empty;
        int clock;

        if (!INSN_P (insn))
@@ -7047,7 +7050,7 @@ reset_sched_cycles_in_current_ebb (void)
  	    haifa_cost = 0;
  	}
        else
-        haifa_cost = estimate_insn_cost (insn, curr_state);
+        haifa_cost = estimate_insn_cost (insn, curr_state, &empty);

        /* Stall for whatever cycles we've stalled before.  */
        after_stall = 0;
@@ -7081,7 +7084,7 @@ reset_sched_cycles_in_current_ebb (void)
                if (!after_stall
                    && real_insn
                    && haifa_cost > 0
-                  && estimate_insn_cost (insn, curr_state) == 0)
+                  && estimate_insn_cost (insn, curr_state, NULL) == 0)
                  break;

                /* When the data dependency stall is longer than the DFA stall,
@@ -7093,7 +7096,7 @@ reset_sched_cycles_in_current_ebb (void)
                if ((after_stall || all_issued)
                    && real_insn
                    && haifa_cost == 0)
-                haifa_cost = estimate_insn_cost (insn, curr_state);
+                haifa_cost = estimate_insn_cost (insn, curr_state, NULL);
              }

  	  haifa_clock += i;
@@ -7125,7 +7128,8 @@ reset_sched_cycles_in_current_ebb (void)
        if (real_insn)
  	{
  	  cost = state_transition (curr_state, insn);
-	  issued_insns++;
+	  if (!empty)
+	    issued_insns++;

            if (sched_verbose >= 2)
  	    {

Comments

Alexander Monakov March 7, 2012, 11:46 a.m. UTC | #1
On Wed, 7 Mar 2012, Andrey Belevantsev wrote:

> Hello,
> 
> This PR is again about insns that are recog'ed as >=0 but do not change the
> processor state.  As explained in
> http://gcc.gnu.org/bugzilla/show_bug.cgi?id=52203#c8, I've tried experimenting
> with an attribute marking those "empty" insns in MD files and asserting that
> all other insns do have reservations.  As this doesn't seem to be interesting,
> I give up with the idea, and the below patch makes sel-sched do exactly what
> the Haifa scheduler does, i.e. do not count such insns against issue_rate when
> modelling clock cycles.
> 
> Tested on ia64 and x86-64, OK for trunk?  No testcase again because of the
> amount of flags needed.
> 
> Andrey
> 
> 2012-03-07  Andrey Belevantsev  <abel@ispras.ru>
> 
> 	PR rtl-optimization/52203
> 	* sel-sched.c (estimate_insn_cost): New parameter pempty.  Adjust
> 	all callers to pass NULL except ...
> 	(reset_sched_cycles_in_current_ebb): ... here, save the value
> 	in new variable 'empty'.  Increase issue_rate only for
> 	non-empty insns.

This is OK.

Thanks.
diff mbox

Patch

diff --git a/gcc/sel-sched.c b/gcc/sel-sched.c
index 2af01ae..2829f60 100644
--- a/gcc/sel-sched.c
+++ b/gcc/sel-sched.c
@@ -4265,9 +4265,10 @@  invoke_aftermath_hooks (fence_t fence, rtx 
best_insn, int issue_more)
    return issue_more;
  }

-/* Estimate the cost of issuing INSN on DFA state STATE.  */
+/* Estimate the cost of issuing INSN on DFA state STATE.  Write to PEMPTY
+   true when INSN does not change the processor state.  */
  static int
-estimate_insn_cost (rtx insn, state_t state)
+estimate_insn_cost (rtx insn, state_t state, bool *pempty)
  {
    static state_t temp = NULL;